﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.IO.Compression;

/*
 *         Progetto: Generatore del database OUI
 * 
 *           Autore: Roberto Fuligni
 *  Ultima modifica: 12/02/2015
 *  
 *      Descrizione: Generatore di metodo C# per la creazione di un dizionario
 *                   contenente il database OUI.
 *                   Chiave -> OUI della scheda di rete
 *                   Valore -> Nome del produttore della scheda di rete
 
 */

namespace GeneratoreDbOUI
{
    class Generatore
    {
        string nomeFile;
        string descrizione;

        public Generatore(string nomeFile, string descrizione)
        {
            this.nomeFile = nomeFile;
            this.descrizione = descrizione;
        }

        public void stampaMetodo()
        {
            const int DIM_GRUPPO = 500;
            int pos = 0;
            string b64 = comprimi(generaDb());

            Console.WriteLine("public Dictionary<string, string> databaseOUI() {");
            Console.WriteLine("\n\t//{0}\n", descrizione);
            Console.WriteLine("\tbyte[] bufferDb;");
            Console.WriteLine("\tDictionary<string, string> d = new Dictionary<string, string>();");
            Console.WriteLine("\tStringBuilder sb = new StringBuilder();");

            while (pos < b64.Length)
            {
                Console.WriteLine("\tsb.Append(\"{0}\");", b64.Substring(pos, Math.Min(b64.Length - pos, DIM_GRUPPO)));
                pos += DIM_GRUPPO;
            }

            Console.WriteLine("\tusing (var cs = new MemoryStream(Convert.FromBase64String(sb.ToString()))) {");
            Console.WriteLine("\t\tusing (var us = new MemoryStream()) {");
            Console.WriteLine("\t\t\tusing (var ds = new DeflateStream(cs, CompressionMode.Decompress)) {");
            Console.WriteLine("\t\t\t\tds.CopyTo(us);");
            Console.WriteLine("\t\t\t\tus.Seek(0, SeekOrigin.Begin);");
            Console.WriteLine("\t\t\t\tbufferDb = us.ToArray();");
            Console.WriteLine("\t\t\t}");
            Console.WriteLine("\t\t}");
            Console.WriteLine("\t}");

            Console.WriteLine("\tusing (BinaryReader r = new BinaryReader(new MemoryStream(bufferDb))) {");
            Console.WriteLine("\t\twhile (r.PeekChar() != -1) {");
            Console.WriteLine("\t\t\tstring chiave = r.ReadString();");
            Console.WriteLine("\t\t\tstring valore = r.ReadString();");
            Console.WriteLine("\t\t\tif (!d.ContainsKey(chiave))");
            Console.WriteLine("\t\t\t\td[chiave] = valore;");
            Console.WriteLine("\t\t}");
            Console.WriteLine("\t}");
            Console.WriteLine("\treturn d;");
            Console.WriteLine("}");
        }

        private byte[] generaDb()
        {
            MemoryStream ris = new MemoryStream();

            using (StreamReader fs = new StreamReader(nomeFile))
            {
                using (BinaryWriter w = new BinaryWriter(ris))
                {

                    string linea;
                    while ((linea = fs.ReadLine()) != null)
                    {
                        if (linea.Length > 0 && linea.IndexOf("(hex)") >= 0)
                        {
                            string[] campi = linea.Split('\t');
                            string chiave = campi[0].Trim().Substring(0, 8);
                            string valore = campi[2];
                            w.Write(chiave);
                            w.Write(valore);
                        }
                    }
                }
            }
            return ris.ToArray();
        }

        private string comprimi(byte[] buffer)
        {
            // Restituisce un buffer compresso in formato base64
            string ris;

            using (var us = new MemoryStream(buffer))
            {
                us.Seek(0, SeekOrigin.Begin);

                using (var cs = new MemoryStream())
                {
                    using (var ds = new DeflateStream(cs, CompressionMode.Compress))
                    {
                        us.CopyTo(ds);
                    }
                    ris = Convert.ToBase64String(cs.ToArray());
                }
            }

            return ris;
        }

        public void Test()
        {
            string b64 = comprimi(generaDb());
            byte[] bufferDb;
            Dictionary<string, string> d = new Dictionary<string,string>();

            using (var cs = new MemoryStream(Convert.FromBase64String(b64)))
            {
                using (var us = new MemoryStream())
                {
                    using (var ds = new DeflateStream(cs, CompressionMode.Decompress))
                    {
                        ds.CopyTo(us);
                        us.Seek(0, SeekOrigin.Begin);

                        bufferDb = us.ToArray();
                    }
                }
            }

            using (BinaryReader r = new BinaryReader(new MemoryStream(bufferDb)))
            {
                while (r.PeekChar() != -1)
                {
                    string chiave = r.ReadString();
                    string valore = r.ReadString();

                    if (!d.ContainsKey(chiave))
                        d[chiave] = valore;
                }
            }

            Console.WriteLine(d["00-00-0F"]);
        }

    }
}
